{-# LANGUAGE FlexibleContexts, TemplateHaskell, TypeFamilies, ScopedTypeVariables, ViewPatterns #-}
module PolicyAlign where
import Generics.BiGUL
import Generics.BiGUL.Interpreter
import Generics.BiGUL.TH
import Generics.BiGUL.Lib
import Generics.BiGUL.Lib.List
import Data.List
import Data.Maybe
import GHC.Generics
import Debug.Trace
import Structure
import PRBAC
import AlignId
import System.IO  
import Control.Monad
import Text.Regex

keymatched :: Policy -> KeyValuePair -> Bool
keymatched pol kv | str1 == str2 = True
                  | otherwise = False
                    where str1 = paradata pol
                          str2 = fst kv

judgearraystring :: [String] -> [String] -> Bool
judgearraystring s1 s2 | length s1 /= length s2 = False
                       | length s1 == 0 = True
                       | (head s1) == (head s2) = judgearraystring (tail s1) (tail s2)
                       | otherwise = False

valuereadsyc :: Policy -> BiGUL Value Value 
valuereadsyc pol = Case
      [
         $(normal [| \ s v -> valuetype s == 1 && valuetype v == 1 && getvaluestring s /= getvaluestring v  |] [| const False |])
              ==> (Fail "the data can only be read not write (string)")
        ,$(normal [| \ s v -> valuetype s == 1 && valuetype v == 1 && getvaluestring s == getvaluestring v  |] 
                  [| \ s -> valuetype s == 1  |])
              ==> Replace
        ,$(normal [| \ s v -> valuetype s == 2 && valuetype v == 2 && 
                     judgeobject (getvalueobject s)(getvalueobject v) == False |]
                  [|  const False |])
              ==> (Fail "the data can only be read not write (object)")
        ,$(normal [| \ s v -> valuetype s == 2 && valuetype v == 2 && 
                     judgeobject (getvalueobject s)(getvalueobject v) == True |]
                  [| \ s -> valuetype s == 2  |])
              ==> Replace
        ,$(normal [| \ s v -> valuetype s == 3 && valuetype v == 3 && 
                     judgearraystring (getvaluearraystring s)(getvaluearraystring v) == False |]
                  [|  const False |])
              ==> (Fail "the data can only be read not write (object)")
        ,$(normal [| \ s v -> valuetype s == 3 && valuetype v == 3 && 
                     judgearraystring (getvaluearraystring s)(getvaluearraystring v) == True |]
                  [| \ s -> valuetype s == 3  |])
              ==> Replace
        ,$(normal [| \ s v -> valuetype s == 4 && valuetype v == 4  |] [|  \ s -> valuetype s == 4 |])
              ==> $(update [p| Arro xs |] [p| Arro xs |] [d| xs = alignread pol |] )
        ,$(normal [| \ s v -> not (valuetype s `elem` [1,2,3,4]) ||  not (valuetype v `elem` [1,2,3,4])  |] [| const False |])
              ==> (Fail "data error")
      ]

valuewritesyc :: Policy -> BiGUL Value Value
valuewritesyc pol = Case
      [
         $(normal [| \ s v -> valuetype s == 1 && valuetype v == 1 |] 
                  [| \ s -> valuetype s == 1  |])
              ==> Replace
        ,$(normal [| \ s v -> valuetype s == 2 && valuetype v == 2 |]
                  [| \ s -> valuetype s == 2  |])
              ==> Replace
        ,$(normal [| \ s v -> valuetype s == 3 && valuetype v == 3 |]
                  [| \ s -> valuetype s == 3  |])
              ==> Replace
        ,$(normal [| \ s v -> valuetype s == 4 && valuetype v == 4  |] [|  \ s -> valuetype s == 4 |])
              ==> $(update [p| Arro xs |] [p| Arro xs |] [d| xs = alignwrite pol |] )
        ,$(normal [| \ s v -> not (valuetype s `elem` [1,2,3,4]) ||  not (valuetype v `elem` [1,2,3,4])  |] [| const False |])
              ==> (Fail "data error")
      ]

datasyc :: Policy -> BiGUL KeyValuePair KeyValuePair
datasyc pol = Case
      [
        $(normal [| \ s v -> paraaction pol == "Write" |] [| \ s -> paraaction pol == "Write" |])
              ==> $(update [p| (x,xs) |] [p| (x,xs) |] [d| x = Replace; xs = valuewritesyc pol |] )
       ,$(normal [| \ s v -> paraaction pol == "Read" |] [| \ s -> paraaction pol == "Read" |])
              ==> $(update [p| (x,xs) |] [p| (x,xs) |] [d| x = Replace; xs = valuereadsyc pol |] )
      ]
        

policysyn :: Policy -> BiGUL Object Object
policysyn pol = Case      
      [
        $(normalSV [p| [] |] [p| [] |] [p| [] |] )
              ==> Replace
       ,$(normal [| \ (s1:ss) (v1:vs) -> not (keymatched pol v1) |] [| const False |])
              ==> (Fail "view is not legal (information is inconsistent with policy)")
       ,$(normal [| \ (s1:ss) (v1:vs) -> keymatched pol s1 && keymatched pol v1|] [| \ (s1:ss) -> keymatched pol s1 |])
              ==> $(update [p| x:xs |] [p| x:xs |] [d| x = datasyc pol ; xs = policysyn pol |] )
       ,$(normal [| \ (s1:ss) vs -> not (keymatched pol s1) |] [| \ (s1:ss) -> not (keymatched pol s1) |])
              ==> $(update [p| _:xs |] [p| xs |] [d| xs = policysyn pol |] )
       ,$(adaptive[| \ss (v1:vs)-> length ss == 0 |])
              ==> \ss (v:_) -> v : ss
      ]       

{-
polexample :: Policy
polexample = Pol (Rol "lab") (DP ("Write","labs")) (Pur "MedicalCare") (Con (("LabDateTime",""),"NEqual")) (ONull)


ns :: [Object] 
ns =  [[("PatientID",Str "FB2ABB23-C9D0-4D09-8464-49BF0B982F0F"),("AdmissionID",Str "7"),("labName",Str "CBC"),("LabValue",Str "1.8"),("LabUnits",Str "pg"),("LabDateTime",Str "")],[("PatientID",Str "64182B95-EB72-4E2B-BE77-8050B71498CE"),("AdmissionID",Str "1"),("labName",Str "METABOLIC"),("LabValue",Str "8.9"),("LabUnits",Str "m/cumm"),("LabDateTime",Str "1992-07-0101")]]

nv :: [Object] 
nv =  [[("PatientID",Str "64182B95-EB72-4E2B-BE77-8050B71498CE"),("AdmissionID",Str "1"),("labName",Str "METABOLIC"),("LabValue",Str "8.9"),("LabUnits",Str "m/cumm"),("LabDateTime",Str "1993-07-0101")]]
-}

